"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.toCodePoints = exports.parse = exports.TypeName = void 0;
const emoji_regex_1 = __importDefault(require("emoji-regex"));
const path_1 = __importDefault(require("path"));
const dataSource = require.resolve("emoji-datasource-apple");
const emojiDir = path_1.default.join(path_1.default.dirname(dataSource), "img", "apple", "64");
const emoji = require("emoji-datasource-apple");
exports.TypeName = "emoji";
function parse(text) {
    const entities = [];
    const regex = (0, emoji_regex_1.default)();
    regex.lastIndex = 0;
    while (true) {
        const result = regex.exec(text);
        if (!result) {
            break;
        }
        const emojiText = result[0];
        const codepoints = toCodePoints(removeVS16s(emojiText)).map((code) => code.toUpperCase());
        let skin = undefined;
        while (codepoints.length > 0) {
            const joined = codepoints.join("-");
            const search = emoji.find((data) => data.unified === joined || data.non_qualified === joined);
            if (search?.has_img_apple) {
                const skinVar = skin ? search.skin_variations?.[skin] : undefined;
                const image = skinVar?.has_img_apple ? skinVar.image : search.image;
                entities.push({
                    url: path_1.default.join(emojiDir, image),
                    indices: [result.index, regex.lastIndex],
                    text: emojiText,
                    type: exports.TypeName,
                });
                break;
            }
            skin = codepoints.pop();
        }
    }
    return entities;
}
exports.parse = parse;
const vs16RegExp = /\uFE0F/g;
// avoid using a string literal like '\u200D' here because minifiers expand it inline
const zeroWidthJoiner = String.fromCharCode(0x200d);
const removeVS16s = (rawEmoji) => rawEmoji.indexOf(zeroWidthJoiner) < 0
    ? rawEmoji.replace(vs16RegExp, "")
    : rawEmoji;
function toCodePoints(unicodeSurrogates) {
    const points = [];
    let char = 0;
    let previous = 0;
    let i = 0;
    while (i < unicodeSurrogates.length) {
        char = unicodeSurrogates.charCodeAt(i++);
        if (previous) {
            points.push((0x10000 + ((previous - 0xd800) << 10) + (char - 0xdc00)).toString(16));
            previous = 0;
        }
        else if (char > 0xd800 && char <= 0xdbff) {
            previous = char;
        }
        else {
            points.push(char.toString(16));
        }
    }
    return points;
}
exports.toCodePoints = toCodePoints;
